@@ -1,9 +1,4 @@
 /* ============================================================
- *
- * This file is part of the KDE project
- *
- * Date        : 2007-09-13
- * Description : Sane interface for KDE
  *
  * SPDX-FileCopyrightText: 2007-2011 Kare Sars <kare.sars@iki .fi>
  * SPDX-FileCopyrightText: 2014 Gregor Mitsch : port to KDE5 frameworks
@@ -25,13 +20,34 @@
 
 // KDE includes
 
-#include <klocalizedstring.h>
+#include <KLocalizedString>
 
 namespace KSaneIface
 {
 
 LabeledEntry::LabeledEntry(QWidget *parent, const QString &ltext)
     : KSaneOptionWidget(parent, ltext)
+{
+    initEntry();
+}
+
+LabeledEntry::LabeledEntry(QWidget *parent, KSaneOption *option)
+    : KSaneOptionWidget(parent, option)
+{
+    initEntry();
+    setToolTip(option->description());
+    setLabelText(option->title());
+    connect(this, &LabeledEntry::entryEdited, option, &KSaneOption::setValue);
+    connect(option, &KSaneOption::valueChanged, this, &LabeledEntry::setValue);
+    QString value = option->value().toString();
+    m_entry->setText(value);
+}
+
+LabeledEntry::~LabeledEntry()
+{
+}
+
+void LabeledEntry::initEntry()
 {
     m_entry = new QLineEdit(this);
     m_reset = new QPushButton(this);
@@ -48,16 +64,20 @@ LabeledEntry::LabeledEntry(QWidget *parent, const QString &ltext)
     connect(m_set, &QPushButton::clicked, this, &LabeledEntry::setClicked);
 }
 
-LabeledEntry::~LabeledEntry()
-{
-}
-
 void LabeledEntry::setText(const QString &text)
 {
     m_eText = text;
     m_entry->setText(text);
 }
 
+void LabeledEntry::setValue(const QVariant &value)
+{
+    const QString text = value.toString();
+    if (!text.isEmpty()) {
+        setText(text);
+    }
+}
+
 void LabeledEntry::resetClicked()
 {
     m_entry->setText(m_eText);
